
package vvasam.jenardf;

import com.hp.hpl.jena.rdf.model.* ;
import com.hp.hpl.jena.rdql.*;
import com.hp.hpl.jena.db.*;
import java.util.* ;
import java.io.* ;
import java.sql.*;
import java.net.*;

import com.hp.hpl.jena.util.iterator.*;



public class SemanticRDFUtils
{

    private static String rdfNameSpace="http://www.w3.org/1999/02/22-rdf-syntax-ns#"; //#type
    private static String rdfsNameSpace="http://www.w3.org/TR/1999/PR-rdf-schema-19990303#"; //label

    private static String mDocRDFNS="http://www.vvasam.net/vvasam#";
    private static Model m = ModelFactory.createDefaultModel();
    private static Property rdfTypeP = m.createProperty(rdfNameSpace, "type" );
    private static Property rdfsLabelP = m.createProperty(rdfsNameSpace, "label" );


	private PropertyFileReader mConfig;

	private Model mModel;
    private  DBConnection dbcon;
    private String mDbUserName;
    private String mDbUserPassword;
    private String mJdbcUrl;
    private String mRdbSystemModelName="JenaRDFSystem";
    private String mModelName;
    private String mConfigRDFFileAbsBasePath;

    private String mInDB="";

    private String mURL="";


    public BufferedReader mSysIn;





    public static final String RDF_NAMESPACE="rdf_namespace";
    public static final String DB_USER_NAME="db_user";
	public static final String DB_USER_PWD="db_password";
	public static final String DB_JDBC_URL="db_jdbc_url";
	public static final String RDF_SYSTEM_MODEL_NAME="rdf_sytem_model_name";

	public static final String MODELNAME="modelName";
	public static final String PROPNAME_URL="url";
	public static final String PROPNAME_ISRDFINDB="isRDFInDB";


	public static final String RDF_QUERY="rdf_query";
	public static final String BIND_VAR_NAME="bind_var_name";
	public static final String IMPORT_RDFFILE_ABS_NAME="import_rdffile_abs_name";
	public static final String EXPORT_RDFFILE_ABS_NAME="export_rdffile_abs_name";
	public static final String FILE_RDFMODEL1_ABS_NAME="file_rdfmodel1_abs_name";
	public static final String FILE_RDFMODEL2_ABS_NAME="file_rdfmodel2_abs_name";
	public static final String RESULT_RDFMODEL_ABS_NAME="result_rdfmodel_abs_name";

	public static final int MODEL_UNION=1;
	public static final int MODEL_INTERSECTION=2;
	public static final int MODEL_DIFFERENCE=3;


    public SemanticRDFUtils(PropertyFileReader aConfig) throws Throwable{
    	if(aConfig==null){
    		 throw new Exception("Invalid config object.");
    	}
		mConfig=aConfig;
		mInDB=mConfig.getPropertyAsString(PROPNAME_ISRDFINDB);
		mURL=mConfig.getProperty(PROPNAME_URL);
		mModelName=mConfig.getProperty(MODELNAME);
		mDocRDFNS=mConfig.getProperty(RDF_NAMESPACE);
		if(!isRDFInDB()){
			mModelName="";
		}
		mRdbSystemModelName=mConfig.getProperty(RDF_SYSTEM_MODEL_NAME);
		mSysIn= new BufferedReader(new InputStreamReader(System.in));
		mModel=openModel();



    }

    public SemanticRDFUtils(String url, String userName, String password)  throws Exception {

        dbcon=createDBConnection(url, userName, password);

    }

    public Model openModel() throws Throwable{
		if(mModel==null){
		   if(isRDFInDB()){
		     if(mModelName!=null && !mModelName.trim().equalsIgnoreCase(""))  {
				mModel=openRDBModel();
		     }else{
		     	//for some of the operations the model need not be initialized at the startup. 
		     	//It can be initialized later ...
		     }
		   }else{
			  mModel=openFileModel();
		   }
		}
		return mModel;

    }

    private void displayMessage(String aMessage){
		System.out.println(aMessage);
    }

	private void displayMessage(String aMessage, boolean newLine){
		if(newLine){
		  System.out.println(aMessage);
		}else{
		  System.out.print(aMessage);
		}

	}



    private void createDBConnection() throws Throwable{
		dbcon = createDBConnection( mConfig.getPropertyAsString(PROPNAME_URL),
		mConfig.getPropertyAsString(DB_USER_NAME), mConfig.getPropertyAsString(DB_USER_PWD));

    }

	public  DBConnection createDBConnection(String url, String username, String password) throws Exception{
	   Class.forName("oracle.jdbc.driver.OracleDriver");
	   displayMessage("  ");
	   Connection con = DriverManager.getConnection(url, username, password);
	   if (null == con) {
		  displayMessage("-->> Failed to create connection");
		  throw new Exception("Failed to create connection");
	   } else {
		 displayMessage("-->> Connection commit setting: " + con.getAutoCommit());
	   }
	   DBConnection dbcon = new DBConnection(con, "Oracle");
	   displayMessage("-->> Established connection to :"+url);
	   displayMessage("  ");
	   return dbcon;
	}



	/**
	 Initializes a Model from a RDF in Oracle Database
	 */
	private Model openRDBModel() throws Throwable {
		if(dbcon==null){
			 createDBConnection();
		}
		ModelRDB model=null;
		if(isModelExists((IDBConnection)dbcon, mModelName)){
			model= ModelRDB.open((IDBConnection)dbcon, mModelName);
		}else{
			throw new Exception(mModelName+" doesn't exist in the database.");
		}

		return (Model)model;
	}

	/**
	 Initializes a Model from a RDF/XML file
	 */
	private Model openFileModel() throws Throwable {

		Model model = ModelFactory.createDefaultModel();
		URL url = new URL(mConfig.getPropertyAsString(PROPNAME_URL));

		// read the RDF/XML file
		//model.read(new InputStreamReader(url.openStream()), ""); //deprecated
		model.read(url.openStream(), "");
		displayMessage("-->> Opened the model using a file :"+mURL);
		displayMessage("  ");


		return model;
	}

	public boolean isRDFInDB(){
		if(mInDB.equalsIgnoreCase("true")){
			return true;
		}else{
			return false;
		}
	}

	public boolean isRDFDMModelInitialized(){
		if(mModel!=null){
			return true;
		}else{
			return false;
		}
	}


    public  void createAndInititalizeJenaTables() throws Throwable{
		if(isRDFInDB() ){
			if(dbcon==null){
  				createDBConnection();
			}
			IDBConnection idb = dbcon;
			boolean modelFound=isModelExists(idb, mRdbSystemModelName);
			if(modelFound){
				displayMessage("----->> The model '"+mRdbSystemModelName +"' already exists. This operation deletes all the models by cleaning the database tables.");
				System.out.print("----->>Do you want to continue(y/n)?");
				String confirm=(mSysIn.readLine()).trim();
				displayMessage("");
				if(confirm!=null && confirm.equalsIgnoreCase("y")){
					displayMessage("----->> Cleaning system tables ....");
					dbcon.cleanDB();
					displayMessage("----->> CleanDB is done.");
					displayMessage("----->> Creating system model :"+mRdbSystemModelName);
					ModelRDB modelRDB = ModelRDB.createModel(idb,mRdbSystemModelName);
					displayMessage("----->> Created system model with name:"+mRdbSystemModelName);
				}else{
				   displayMessage("----->> Operation cancelled.");
				   return;
				}
			}else{
				displayMessage("----->> Creating system model :"+mRdbSystemModelName);
				ModelRDB modelRDB = ModelRDB.createModel(idb,mRdbSystemModelName);
				mRdbSystemModelName=mRdbSystemModelName;
				displayMessage("----->> Created system model with name:"+mRdbSystemModelName);
			}
		}else{
			throw new RuntimeException("This service is not for file RDF models.");
		}
    }


    public void cleanRDFDB() throws Throwable{
		if(isRDFInDB() && dbcon==null){
			createDBConnection();
            dbcon.cleanDB();
		}else{
			throw new RuntimeException("This service is not for file RDF models.");
		}

    }



    public void  close() throws Exception{
       try{
       	if(dbcon!=null){
            dbcon.close();
       	}
       	if(mModel!=null){
			mModel.close();
       	}
       	if(mSysIn!=null){
       		mSysIn.close();
       	}
       }finally{
            dbcon=null;
			mModel=null;
			mSysIn=null;
       }
    }

    public   void listExistingModels() throws Throwable{
    	if(!isRDFInDB()){
    		throw new RuntimeException("This service is not for file RDF models.");
    	}else{
		   if(dbcon==null){
			createDBConnection();
		   }
	       ExtendedIterator ei=ModelRDB.listModels(dbcon);
	       int i=0;
	       while(ei.hasNext()){
	        i++;
	        Object o=ei.next();
	        displayMessage("----->> Model :"+i+": "+o);
	       }
    	}
    }

    public  void createRDBModel(String aModelName) throws Throwable{
		if(!isRDFInDB()){
				throw new RuntimeException("This service is not for file RDF models.");
		}else{
			if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
						throw new IllegalArgumentException("Invalid model name");
			}
			if(dbcon==null){
		 		createDBConnection();
			}
			IDBConnection idb = dbcon;
			boolean modelFound=isModelExists(idb, aModelName);
			if(modelFound){
				displayMessage("----->> The model '"+aModelName +"' already exists.");
				return;
			}

	        com.hp.hpl.jena.db.impl.IRDBDriver driver=idb.getDriver();
	        if(driver!=null){
	          displayMessage("----->> Setting store with model :"+mRdbSystemModelName);
	          driver.setStoreWithModel(mRdbSystemModelName);
	        }else{
	            throw new Exception("There is no system model with name :"+mRdbSystemModelName);
	        }
	        ModelRDB modelRDB = ModelRDB.createModel(idb,aModelName);
	        displayMessage("----->> Created empty model with name:"+aModelName);
		}
    }

     public  void removeModel(String aModelName) throws Throwable{
		if(!isRDFInDB()){
				throw new RuntimeException("This service is not for file RDF models.");
		}else{
			if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
						throw new IllegalArgumentException("Invalid model name");
			}
			if(dbcon==null){
				createDBConnection();
			}
			IDBConnection idb = dbcon;
			boolean systemModelFag=false;
	        if(mRdbSystemModelName.equalsIgnoreCase(aModelName.trim())){
	        	systemModelFag=true;
				/*displayMessage("----->> The model '"+aModelName +"'is a System model. This operation deletes all the models by cleaning the database tables.");
				System.out.print("----->>Do you want to continue(y/n)?");
				String confirm=(mSysIn.readLine()).trim();
				displayMessage("");
				if(confirm!=null && !confirm.equalsIgnoreCase("y")){
				   displayMessage("----->> Operation cancelled.");
				   return;
				}
				*/
        	}
	        boolean modelFound=isModelExists(idb, aModelName);
	        if(modelFound){
	         try{
				boolean confirm= false;
				if(systemModelFag){
					confirm=getConfirmation("----->>The model '"+aModelName +"'is a System model. This operation deletes all the models by cleaning the database tables.");
				}else{
					confirm=getConfirmation("----->> Deleting model with name :"+aModelName+".");
				}
	            if(confirm==true){
					ModelRDB modelRDB = ModelRDB.open(idb, aModelName);
	                modelRDB.remove();
	            }else{
	               displayMessage("----->> Remove cancelled.");
	               return;
	            }
	         }catch(NullPointerException ne){
	          /* If the user don't have the schema level privileges then the NullPointerException would be thrown
	          after deleting the model that uses system tables. This is a normal behaviour.
	          */
	         }catch(Exception ex){
	             throw ex;
	         }
	         displayMessage("----->> Removed model with name:"+aModelName);
	        }else{
	         displayMessage("----->> No model available with name:"+aModelName);
	        }
		}
     }

      public  boolean isModelExists(IDBConnection aIDBConn,String aModelName) throws Exception{
        if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
            throw new IllegalArgumentException("Invalid model name");
        }
        if(aIDBConn==null){
         throw new IllegalArgumentException("IDBConnection is null");
        }
        boolean modelFound=false;
        ExtendedIterator ei=ModelRDB.listModels(aIDBConn);
        while(ei.hasNext()){
           Object obj=ei.next();
           if( obj instanceof String){
              String modelName=(String)obj;
              if(modelName.equalsIgnoreCase(aModelName)){
                   modelFound=true;
                   break;
              }
           }else{
               throw new RuntimeException("Unknown model object :");
           }
        }
        if(modelFound){
          return true;
        }else{
           return false;
        }

    }

    public  Model importRDFXMLFile(String aModelName,String aModelAbsFileName )   throws Throwable{
		if(!isRDFInDB()){
				throw new RuntimeException("This service is not for file RDF models.");
		}else{
			if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
						throw new IllegalArgumentException("Invalid model name");
			}
			if(aModelAbsFileName==null ||(aModelAbsFileName!=null && aModelAbsFileName.trim().equalsIgnoreCase(""))){
			 throw new IllegalArgumentException("Invalid model RDF file name");
			}
			if(dbcon==null){
				createDBConnection();
			}
			IDBConnection idb = dbcon;
            ModelRDB modelRDB = ModelRDB.open(idb,aModelName);
            if(modelRDB!=null){
				displayMessage("----->> Opened existing  model for eai.");
    			Model model = ModelFactory.createDefaultModel();
				InputStream in = new FileInputStream(aModelAbsFileName);
				if (in == null) {
					throw new IllegalArgumentException(
						 "File: " + aModelAbsFileName + " not found");
				}
				// read the RDF/XML file
				//model.read(new InputStreamReader(in), ""); //Deprecated....
				model.read(in, "");
                displayMessage("----->> adding model: "+aModelAbsFileName);
				modelRDB.add(model);
                displayMessage("----->> added model");

             }
            return modelRDB;
		}
      }

	  public  void deleteModelContents()   throws Throwable {
	  	if(mModel!=null){
	  		try{
			   boolean confirm=getConfirmation("----->> Deleting model with name :"+mModelName+".");
			   if(confirm==true){
				   StmtIterator sit=mModel.listStatements();
				   Vector stmts=new Vector();
				   while(sit.hasNext()){
					   com.hp.hpl.jena.rdf.model.Statement st=(com.hp.hpl.jena.rdf.model.Statement)sit.nextStatement();
					   stmts.add(st);
				   }
				   com.hp.hpl.jena.rdf.model.Statement [] sts=new com.hp.hpl.jena.rdf.model.Statement[stmts.size()];
				   sts=(com.hp.hpl.jena.rdf.model.Statement[])stmts.toArray(sts);
				   mModel.remove(sts);
			   }else{
				   displayMessage("----->> Delete cancelled.");
				   return;
			   }

			}catch(NullPointerException ne){
			 /* If the user don't have the schema level privileges then the NullPointerException would be thrown
			 after deleting the model that uses system tables. This is a normal behaviour.
			 */
			}catch(Exception ex){
				throw ex;
			}
			displayMessage("----->> Deleted all the contents of the model :"+mModelName);
   	   }else{
		   displayMessage("----->> Model is not initialized..");
   	   }
	  }

	  public  void deleteModelContents(String aModelName )   throws Throwable {
		  if(!isRDFInDB()){
				throw new RuntimeException("This service is not for file RDF models.");
		  }else{
			if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
						throw new IllegalArgumentException("Invalid model name");
			}
			if(dbcon==null){
				createDBConnection();
			}
			IDBConnection idb = dbcon;
			boolean modelFound=isModelExists(idb, aModelName);
			if(modelFound){
				 ModelRDB modelRDB = ModelRDB.open(idb, aModelName);
				 try{
					boolean confirm=getConfirmation("----->> Deleting model with name :"+aModelName+".");
					if(confirm==true){
						StmtIterator sit=modelRDB.listStatements();
						Vector stmts=new Vector();
						while(sit.hasNext()){
							com.hp.hpl.jena.rdf.model.Statement st=(com.hp.hpl.jena.rdf.model.Statement)sit.nextStatement();
							stmts.add(st);
						}
						com.hp.hpl.jena.rdf.model.Statement [] sts=new com.hp.hpl.jena.rdf.model.Statement[stmts.size()];
						sts=(com.hp.hpl.jena.rdf.model.Statement[])stmts.toArray(sts);
						modelRDB.remove(sts);
					}else{
						displayMessage("----->> Delete cancelled.");
   						return;
					}

				 }catch(NullPointerException ne){
				  /* If the user don't have the schema level privileges then the NullPointerException would be thrown
				  after deleting the model that uses system tables. This is a normal behaviour.
				  */
				 }catch(Exception ex){
					 throw ex;
				 }
				 displayMessage("----->> Deleted all the contents of the model :"+aModelName);
			}else{
				 displayMessage("----->> No model available with name:"+aModelName);
			}
		  }
	  }


	  public  long getStmtCount( )   throws Throwable {
		  	long count=0;
			Vector stmts=new Vector();
			if(mModel!=null){
			  	try{
				  StmtIterator sit=mModel.listStatements();
				   while(sit.hasNext()){
					  com.hp.hpl.jena.rdf.model.Statement st=(com.hp.hpl.jena.rdf.model.Statement)sit.nextStatement();
					  stmts.add(st);
				  }
				}catch(Exception ex){
				   throw ex;
				}
				count=stmts.size();
			}else{
		  		displayMessage("----->> Model is not initialized..");
	  		}
		    return count;
	  }

	  public  long getStmtCount(String aModelName )   throws Throwable {
			long count=0;
			if(!isRDFInDB()){
					throw new RuntimeException("This service is not for file RDF models.");
			}else{
				if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
							throw new IllegalArgumentException("Invalid model name");
				}
				if(dbcon==null){
					createDBConnection();
				}
				IDBConnection idb = dbcon;
				boolean modelFound=isModelExists(idb, aModelName);
				if(modelFound){
					ModelRDB modelRDB = ModelRDB.open(idb, aModelName);
					Vector stmts=new Vector();
					 try{
						StmtIterator sit=modelRDB.listStatements();

						while(sit.hasNext()){
							com.hp.hpl.jena.rdf.model.Statement st=(com.hp.hpl.jena.rdf.model.Statement)sit.nextStatement();
							stmts.add(st);
						}
					 }catch(Exception ex){
						 throw ex;
					 }
					count=stmts.size();
				}else{
					 displayMessage("----->> No model available with name:"+aModelName);
					 throw new Exception("No model available with name"+aModelName);
				}
			}
			return count;
		 }

		public void saveGraphOperationResultModelToFile(int aGraphOperation) throws Throwable{

			String firstRDFModelAbsFileName = getProperty(FILE_RDFMODEL1_ABS_NAME);
			displayMessage("--> "+FILE_RDFMODEL1_ABS_NAME+":"+firstRDFModelAbsFileName);

			String secondRDFModelAbsFileName =  getProperty(FILE_RDFMODEL2_ABS_NAME);
			displayMessage("--> "+FILE_RDFMODEL2_ABS_NAME+":"+secondRDFModelAbsFileName);

			String resultRDFModelAbsFileName =  getProperty(RESULT_RDFMODEL_ABS_NAME);
			displayMessage("--> "+RESULT_RDFMODEL_ABS_NAME+" :"+resultRDFModelAbsFileName);

			saveGraphOperationResultModelToFile(aGraphOperation,firstRDFModelAbsFileName,secondRDFModelAbsFileName,resultRDFModelAbsFileName);

		}
		public void saveGraphOperationResultModelToFile(int aGraphOperation,String firstRDFModelAbsFileName, String secondRDFModelAbsFileName,String resultRDFModelAbsFileName) throws Throwable{

			if(firstRDFModelAbsFileName==null || (firstRDFModelAbsFileName!=null && firstRDFModelAbsFileName.trim().equalsIgnoreCase(""))){
				throw new Exception("Invalid first model abs file name.");
			}

			if(secondRDFModelAbsFileName==null || (secondRDFModelAbsFileName!=null && secondRDFModelAbsFileName.trim().equalsIgnoreCase(""))){
				throw new Exception("Invalid second model abs file name.");
			}

			if(resultRDFModelAbsFileName==null || (resultRDFModelAbsFileName!=null && resultRDFModelAbsFileName.trim().equalsIgnoreCase(""))){
				throw new Exception("Invalid result model abs file name.");
			}

			if(!resultRDFModelAbsFileName.endsWith(".rdf")){
				resultRDFModelAbsFileName=resultRDFModelAbsFileName+".rdf";
			}
			//first
			Model model1 = ModelFactory.createDefaultModel();
			InputStream in1 = new FileInputStream(firstRDFModelAbsFileName);
			if (in1 == null) {
				throw new IllegalArgumentException(
						 "File: " + firstRDFModelAbsFileName + " not found");
			}
			// read the RDF/XML file
			model1.read(new InputStreamReader(in1), "");

			//second
			Model model2 = ModelFactory.createDefaultModel();
			InputStream in2 = new FileInputStream(secondRDFModelAbsFileName);
			if (in2 == null) {
				throw new IllegalArgumentException(
									 "File: " + secondRDFModelAbsFileName + " not found");
			}
			// read the RDF/XML file
			model2.read(new InputStreamReader(in2), "");
			Model result=null;
			switch(aGraphOperation){
				case MODEL_UNION:{
					result=model1.union(model2);
					break;
				}
				case MODEL_INTERSECTION:{
					result=model1.intersection(model2);
					break;
				}

				case MODEL_DIFFERENCE:{
					result=model1.difference(model2);
					break;
				}

				default:{
					throw new Exception(aGraphOperation+" is a invalid operation.");

				}



			}

			BufferedOutputStream bo=new  BufferedOutputStream(new FileOutputStream(resultRDFModelAbsFileName));
			displayMessage("-->Writing to a file :"+resultRDFModelAbsFileName);
			result.write(bo,"RDF/XML");
			displayMessage("-->Created file :"+resultRDFModelAbsFileName);
			bo.close();
			result.close();
			model1.close();
			model2.close();
		}



		public  void printRDBModel()  throws Throwable {
			if(mModel!=null){
			   String queryString = "SELECT * " + " WHERE (?x, ?y, ?z) ";
				//            " USING vvasam  FOR <" + mDocRDFNS + ">" ;
			   Query query = new Query(queryString);
			   query.setSource(mModel);
				QueryExecution qe = new QueryEngine(query) ;
				QueryResults results = qe.exec() ;
				int total=0;
				for ( Iterator iter = results ; iter.hasNext() ; )
				{
					ResultBinding res = (ResultBinding)iter.next() ;
					Object x = res.get("x") ;
					Object y = res.get("y");
					System.out.print(" x = " + x);
					System.out.print(" y = " + y);
					Object z = res.get("z") ;
					displayMessage("  z = " + z);
					total++;
				}
				displayMessage("----->> Total is :"+total);
			} else{
				displayMessage("----->> Model is not initialized..");
			}
		 }


      	public  void printRDBModel(String aModelName)  throws Throwable {
			if(!isRDFInDB()){
					throw new RuntimeException("This service is not for file RDF models.");
			}else{
				if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
							throw new IllegalArgumentException("Invalid model name");
				}
				if(dbcon==null){
					createDBConnection();
				}
				IDBConnection idb = dbcon;
            	if(isModelExists(idb, aModelName)){
	                ModelRDB modelRDB = ModelRDB.open(idb,aModelName);
	                if(modelRDB!=null){
	                   String queryString = "SELECT * " + " WHERE (?x, ?y, ?z) ";
	                    //            " USING vvasam FOR <" + mDocRDFNS + ">" ;
	                   Query query = new Query(queryString);
	                   query.setSource(modelRDB);
	                    QueryExecution qe = new QueryEngine(query) ;
	                    QueryResults results = qe.exec() ;
	                    int total=0;
	                    for ( Iterator iter = results ; iter.hasNext() ; )
	                    {
	                        ResultBinding res = (ResultBinding)iter.next() ;
	                        Object x = res.get("x") ;
	                        Object y = res.get("y");
	                        System.out.print(" x = " + x);
	                        System.out.print(" y = " + y);
	                        Object z = res.get("z") ;
	                        displayMessage("  z = " + z);
	                        total++;
	                    }
	                    displayMessage("----->> Total is :"+total);
	                 }else{
	                    throw new Exception("Unable to open model ...");
	                }
	           }else{
	              displayMessage("----->> Model doesn't exist..");
	           }
			}
     }

	 public void exportRDFDBModelToRDFXMLFile(String aAbsFileName) throws Throwable{
		if(mModel!=null){
		  if(!aAbsFileName.endsWith(".rdf")){
			  aAbsFileName=aAbsFileName+".rdf";
		  }
		  BufferedOutputStream bo=new  BufferedOutputStream(new FileOutputStream(aAbsFileName));
		  displayMessage("-->Writing to a file :"+aAbsFileName);
		  mModel.write(bo,"RDF/XML");
		  displayMessage("-->Created file :"+aAbsFileName);
		  bo.close();
		}else{
			displayMessage("----->> Model is not initialized..");
		}
	 }

     /**
      * @param aModelName
      * @param aAbsFileName
      * @throws Exception
      */
	 public void exportRDFDBModelToRDFXMLFile(String aModelName, String aAbsFileName) throws Throwable{
		if(!isRDFInDB()){
				throw new RuntimeException("This service is not for file RDF models.");
		}else{
			if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
						throw new IllegalArgumentException("Invalid model name");
			}
			if(dbcon==null){
				createDBConnection();
			}
			IDBConnection idb = dbcon;
			 if(!aAbsFileName.endsWith(".rdf")){
			   aAbsFileName=aAbsFileName+".rdf";
			 }
			 if(isModelExists(idb, aModelName)){
				ModelRDB modelRDB = ModelRDB.open(idb,aModelName);
				if(modelRDB!=null){
				  Model aModel= (Model)modelRDB;
				  BufferedOutputStream bo=new  BufferedOutputStream(new FileOutputStream(aAbsFileName));
				  displayMessage("-->Writing to a file :"+aAbsFileName);
				  aModel.write(bo,"RDF/XML");
				  displayMessage("-->Created file :"+aAbsFileName);
				  bo.close();
				}
			}else{
				 throw new Exception("There is no model with name :"+aModelName);
			}
		}


	 }

	public long getNewURIID(){
		return System.currentTimeMillis();
	}


	public void exportRDFQueryResultsAsRDFXMLFile(String aAbsFileName, String aRDFQuery,String aBindVarName) throws Throwable{
		if(aRDFQuery!=null && aRDFQuery.trim().equalsIgnoreCase("")){
		   throw new Exception("Invalid RDF query. The rdf_query property in properties file either doesn't exist or empty.");
		}

		if(aBindVarName!=null && aBindVarName.trim().equalsIgnoreCase("")){
		   throw new Exception("Invalid bind_var_name. The bind_var_name property in properties file either doesn't exist or empty.");
		}

		if(!aAbsFileName.endsWith(".rdf")){
			  aAbsFileName=aAbsFileName+".rdf";
		}
	    if(mModel!=null){
			 Model aModel= (Model)mModel;
			 Model memMod1 = ModelFactory.createDefaultModel();
			 Model memMod2 = ModelFactory.createDefaultModel();
			 Resource[] res=getResource(aRDFQuery,aBindVarName,aModel);
			 memMod1.add(aModel);
			 memMod2.add(aModel);
			 Vector stmts=new Vector();
			 Vector leafResources=new Vector();
			 if(res!=null & res.length>0){
				displayMessage("Exporting data...........");
				for(int i=0;i<res.length;i++){
					getAllSatementsLinkedToResourceAsSubject(memMod1,memMod2,res[i],stmts,false,leafResources);
				}
				com.hp.hpl.jena.rdf.model.Statement st[]=new com.hp.hpl.jena.rdf.model.Statement[stmts.size()];
				st=(com.hp.hpl.jena.rdf.model.Statement[])stmts.toArray(st);
				Model mem = ModelFactory.createDefaultModel();
				mem.add(st);
				BufferedOutputStream bo=new  BufferedOutputStream(new FileOutputStream(aAbsFileName));
				displayMessage("-->Writing to a file :"+aAbsFileName);
				mem.write(bo,"RDF/XML");
				displayMessage("-->Created file :"+aAbsFileName);
				bo.close();
				aModel.close();
				displayMessage("");
				//displayMessage(" RDF statements  size:"+st.length);


			 }else{
				displayMessage("No data...........");
			 }
		}else{
		   throw new Exception("Model is not initialized.");
		}

	}


	public void exportRDFQueryResultsAsRDFXMLFile(String aModelName, String aAbsFileName, String aRDFQuery,String aBindVarName) throws Throwable{
		if(!isRDFInDB()){
				throw new RuntimeException("This service is not for file RDF models.");
		}else{
			if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
						throw new IllegalArgumentException("Invalid model name");
			}
			if(dbcon==null){
				createDBConnection();
			}
			if(aRDFQuery!=null && aRDFQuery.trim().equalsIgnoreCase("")){
			   throw new Exception("Invalid RDF query. The rdf_query property in properties file either doesn't exist or empty.");
			}

			if(aBindVarName!=null && aBindVarName.trim().equalsIgnoreCase("")){
			   throw new Exception("Invalid bind_var_name. The bind_var_name property in properties file either doesn't exist or empty.");
			}

			if(!aAbsFileName.endsWith(".rdf")){
				  aAbsFileName=aAbsFileName+".rdf";
			}
			IDBConnection idb = dbcon;
			if(isModelExists(idb, aModelName)){
			   ModelRDB modelRDB = ModelRDB.open(idb,aModelName);
			   if(modelRDB!=null){
				 Model aModel= (Model)modelRDB;
				 Model memMod1 = ModelFactory.createDefaultModel();
				 Model memMod2 = ModelFactory.createDefaultModel();
				 Resource[] res=getResource(aRDFQuery,aBindVarName,aModel);
				 memMod1.add(aModel);
				 memMod2.add(aModel);
				 Vector stmts=new Vector();
				 Vector leafResources=new Vector();
				 if(res!=null & res.length>0){
					displayMessage("Exporting data...........");
				 	for(int i=0;i<res.length;i++){
						getAllSatementsLinkedToResourceAsSubject(memMod1,memMod2,res[i],stmts,false,leafResources);
				 	}
					com.hp.hpl.jena.rdf.model.Statement st[]=new com.hp.hpl.jena.rdf.model.Statement[stmts.size()];
					st=(com.hp.hpl.jena.rdf.model.Statement[])stmts.toArray(st);
					Model mem = ModelFactory.createDefaultModel();
					mem.add(st);
					BufferedOutputStream bo=new  BufferedOutputStream(new FileOutputStream(aAbsFileName));
					displayMessage("-->Writing to a file :"+aAbsFileName);
					mem.write(bo,"RDF/XML");
					displayMessage("-->Created file :"+aAbsFileName);
					bo.close();
					aModel.close();
					displayMessage("");
					//displayMessage(" RDF statements  size:"+st.length);


				 }else{
					displayMessage("No data...........");
				 }
			  }else{
				throw new Exception("Null model .");
			  }

			}else{
			   throw new Exception("There is no model with name :"+aModelName);
			}
		}

	}


	public void deleteRDFQueryResultsFromModel(String aQuery, String aBindVar) throws Throwable{
		if(mModel!=null){
		 Model memMod1 = ModelFactory.createDefaultModel();
		 Model memMod2 = ModelFactory.createDefaultModel();
		 Resource[] res=getResource(aQuery,aBindVar,mModel);
		 memMod1.add(mModel);
		 memMod2.add(mModel);
		 Vector stmts=new Vector();
		 Vector leafResources=new Vector();
		 if(res!=null & res.length>0){
			displayMessage("Found resources length:"+res.length);
			for(int i=0;i<res.length;i++){
				Resource resrc=(Resource)res[i];
				displayMessage("Resource :"+resrc);
				if(!isResourceAsObjectShared(mModel,resrc)){
					getSatementsLinkedToResourceAsSubject(mModel,resrc,stmts);
				}

			}
		}
		boolean confirm=getConfirmation("----->>Deleting resource(s) .Do you want to continue(y/n)?");
		if(confirm==false){
			displayMessage("----->> Delete cancelled.");
			return;
		}

		displayMessage("Deleting resource(s) .....");
		com.hp.hpl.jena.rdf.model.Statement st[]=new com.hp.hpl.jena.rdf.model.Statement[stmts.size()];
		st=(com.hp.hpl.jena.rdf.model.Statement[])stmts.toArray(st);
		//
		memMod1.remove(st);  //Todo :delete this stmt
		memMod2.remove(st); //Todo :delete this stmt
		mModel.remove(st);
		displayMessage("Deleted resource(s) data successfully.");
		displayMessage("");
		displayMessage("Deleted statements count:" +st.length);
	 }else{
		throw new Exception("Model is not initialized.");
	 }
	}


	public void deleteRDFQueryResultsFromModel(String aModelName,String aRDFQuery, String aBindVarName) throws Throwable{
		if(!isRDFInDB()){
				throw new RuntimeException("This service is not for file RDF models.");
		}else{
			if(aModelName==null ||(aModelName!=null && aModelName.trim().equalsIgnoreCase(""))){
						throw new IllegalArgumentException("Invalid model name");
			}
			if(dbcon==null){
				createDBConnection();
			}

			if(aRDFQuery!=null && aRDFQuery.trim().equalsIgnoreCase("")){
			   throw new Exception("Invalid RDF query. The rdf_query property in properties file either doesn't exist or empty.");
			}

			if(aBindVarName!=null && aBindVarName.trim().equalsIgnoreCase("")){
			   throw new Exception("Invalid bind_var_name. The bind_var_name property in properties file either doesn't exist or empty.");
			}


			IDBConnection idb = dbcon;
			if(isModelExists(idb, aModelName)){
			   ModelRDB modelRDB = ModelRDB.open(idb,aModelName);
			   if(modelRDB!=null){
				 Model aModel= (Model)modelRDB;
				 Model memMod1 = ModelFactory.createDefaultModel();
				 Model memMod2 = ModelFactory.createDefaultModel();
				 Resource[] res=getResource(aRDFQuery,aBindVarName,aModel);
				 memMod1.add(aModel);
				 memMod2.add(aModel);
				 Vector stmts=new Vector();
				 Vector leafResources=new Vector();
				 if(res!=null & res.length>0){
				 	displayMessage("Found resources length:"+res.length);
					for(int i=0;i<res.length;i++){
						Resource resrc=(Resource)res[i];
						displayMessage("Resource :"+resrc);
						if(!isResourceAsObjectShared(aModel,resrc)){
							getSatementsLinkedToResourceAsSubject(aModel,resrc,stmts);
						}

					}
				}
				boolean confirm=getConfirmation("----->>Deleting resource(s).");
				if(confirm==false){
					displayMessage("----->> Delete cancelled.");
					return;
				}

				displayMessage("Deleting resource(s) .....");
				com.hp.hpl.jena.rdf.model.Statement st[]=new com.hp.hpl.jena.rdf.model.Statement[stmts.size()];
				st=(com.hp.hpl.jena.rdf.model.Statement[])stmts.toArray(st);
				//
				memMod1.remove(st);  //Todo :delete this stmt
				memMod2.remove(st); //Todo :delete this stmt
				aModel.remove(st);
				displayMessage("Deleted resource(s) data successfully.");
				displayMessage("");
				displayMessage("Deleted statements count:" +st.length);
			  }else{
				 throw new Exception("Null model .");
			  }

			}else{
				throw new Exception("There is no model with name :"+aModelName);
			}
		}
	}


	private void getAllSatementsLinkedToResourceAsSubject(Model aModel, Model aModelClone,Resource aSubject, Vector aStmts,boolean aCheckResourceShareStaus,Vector aLeafResources) throws Exception{
		//recursionCounter++;
		String queryString =
						"SELECT ?sproperty,?sobject WHERE (<"+aSubject.getURI()+">,?sproperty, ?sobject)" +
						"USING vvasam FOR <" + mDocRDFNS + ">" ;
		//displayMessage("Query String :"+queryString);

		Query query = new Query(queryString);
		query.setSource(aModel);
		QueryExecution qe = new QueryEngine(query);
		QueryResults results = qe.exec();
		while(results.hasNext()) {
			ResultBinding res = (ResultBinding)results.next() ;
			Value sproperty = res.getValue("sproperty");
			//displayMessage("Property :"+sproperty);

			Property spProp=null;
			if(sproperty.isRDFResource()){
				Resource spResource=sproperty.getRDFResource();
				spProp=m.createProperty(spResource.getNameSpace(),spResource.getLocalName());
			}else{
				throw new Exception("Invalid property type in the model");
			}

			Value sobject = res.getValue("sobject");
			//displayMessage("Object  :"+sobject);
			//displayMessage("Subject :"+aSubject+" property :"+sproperty+" object :"+sobject);
			RDFNode spRDFNode=null;
			if(sobject.isRDFResource()){
				//displayMessage(" object :"+sobject+"  is a resource :");
				Resource resource=sobject.getRDFResource();
				//getAllSatementsLinkedToResourceAsSubject(aModel,resource,aStmts);
				if(aCheckResourceShareStaus){
					//this code would be executed for getting resources that are not shared
					// so that they can be deleted as a part of investment delete operation.
					if(!isResourceAsObjectShared(aModelClone,resource,sproperty.toString())){
						//displayMessage(" object : "+resource+"  is not shared on property :"+sproperty);
						getAllSatementsLinkedToResourceAsSubject(aModel, aModelClone,resource,aStmts,true,aLeafResources);
					}else{
						/* find if the resource is a leaf( The resource with no other resources as objects)
						 *  and store it in a leaf resource vector
						 */
						 if(isLeafResource(aModelClone,resource)){
							RDFNode rnObj=(RDFNode)resource;
							com.hp.hpl.jena.rdf.model.Statement stobj=aModel.createStatement(aSubject,spProp,rnObj); //To do
							if(!isAlreadyExists(aLeafResources,stobj)){
								aLeafResources.add(stobj);
							}
						 }
					}
				}else{
					getAllSatementsLinkedToResourceAsSubject(aModel, aModelClone,resource,aStmts,false,aLeafResources);
				}
				spRDFNode=(RDFNode)resource;
			}else if(sobject.isRDFLiteral()){
				//displayMessage(" object :"+sobject+"  is a Literal :");
				Literal lit=sobject.getRDFLiteral();
				spRDFNode=(RDFNode)lit;
			}


			com.hp.hpl.jena.rdf.model.Statement st=aModel.createStatement(aSubject,spProp,spRDFNode); //To do
			if(st!=null){
				aStmts.add(st);
			}

		}//while(results.hasNext())
		results.close();
		qe.close();
		//displayMessage(aSubject+" iteration is over.>>>>>>>>>>>>>>>>>>>> :"+recursionCounter);

	}


	private void getSatementsLinkedToResourceAsSubject(Model aModel,Resource aSubject, Vector aStmts) throws Exception{
			//recursionCounter++;
			String queryString =
							"SELECT ?sproperty,?sobject WHERE (<"+aSubject.getURI()+">,?sproperty, ?sobject)" +
							"USING vvasam FOR <" + mDocRDFNS + ">" ;
			//displayMessage("Query String :"+queryString);

			Query query = new Query(queryString);
			query.setSource(aModel);
			QueryExecution qe = new QueryEngine(query);
			QueryResults results = qe.exec();
			while(results.hasNext()) {
				ResultBinding res = (ResultBinding)results.next() ;
				Value sproperty = res.getValue("sproperty");
				//displayMessage("Property :"+sproperty);

				Property spProp=null;
				if(sproperty.isRDFResource()){
					Resource spResource=sproperty.getRDFResource();
					spProp=m.createProperty(spResource.getNameSpace(),spResource.getLocalName());
				}else{
					throw new Exception("Invalid property type in the model");
				}

				Value sobject = res.getValue("sobject");
				//displayMessage("Object  :"+sobject);
				//displayMessage("Subject :"+aSubject+" property :"+sproperty+" object :"+sobject);
				RDFNode spRDFNode=null;
				if(sobject.isRDFResource()){
					//displayMessage(" object :"+sobject+"  is a resource :");
					Resource resource=sobject.getRDFResource();
					spRDFNode=(RDFNode)resource;
				}else if(sobject.isRDFLiteral()){
					//displayMessage(" object :"+sobject+"  is a Literal :");
					Literal lit=sobject.getRDFLiteral();
					spRDFNode=(RDFNode)lit;
				}
				com.hp.hpl.jena.rdf.model.Statement st=aModel.createStatement(aSubject,spProp,spRDFNode);
				if(st!=null){
					aStmts.add(st);
				}

			}//while(results.hasNext())
			results.close();
			qe.close();
	}

	private boolean  isAlreadyExists(Vector aLeafResources,com.hp.hpl.jena.rdf.model.Statement stObj ) throws Exception{
		boolean exists=false;
		String propURI1=stObj.getPredicate().getURI();
		String objURI1="";
		RDFNode nd1=stObj.getObject();
		if(nd1 instanceof Resource){
		  objURI1=((Resource)nd1).getURI();
		}else{
		  throw new Exception("Invalid object type.");
		}
		for(int i=0;i<aLeafResources.size();i++){
		  com.hp.hpl.jena.rdf.model.Statement st=(com.hp.hpl.jena.rdf.model.Statement)aLeafResources.elementAt(i);
		  if(st!=null){
			String propURI=st.getPredicate().getURI();
			String objURI="";
			RDFNode nd=st.getObject();
			if(nd instanceof Resource){
				objURI=((Resource)nd).getURI();
			}else{
				throw new Exception("Invalid object type.");
			}
			if(propURI1.trim().equalsIgnoreCase(propURI.trim()) &&
				objURI1.trim().equalsIgnoreCase(objURI.trim())
			  ){
				exists=true;
				return exists;
			}
		  }
		}
		return exists;
	}

	private boolean isLeafResource(Model aModel,Resource aSubject){
		String queryString =
							"SELECT ?sproperty,?sobject WHERE (<"+aSubject.getURI()+">,?sproperty, ?sobject)" +
							"USING vvasam FOR <" + mDocRDFNS + ">" ;
		//displayMessage("isLeafResource Query String :"+queryString);

		Query query = new Query(queryString);
		query.setSource(aModel);
		QueryExecution qe = new QueryEngine(query);
		QueryResults results = qe.exec();
		while(results.hasNext()) {
			ResultBinding res = (ResultBinding)results.next() ;
			Value sproperty = res.getValue("sproperty");
			//displayMessage("Property :"+sproperty);
			Value sobject = res.getValue("sobject");
			//displayMessage("Object  :"+sobject);
			RDFNode spRDFNode=null;
			if(sobject.isRDFResource() && !(rdfTypeP.getURI().trim().equalsIgnoreCase(sproperty.getURI()))){
				//displayMessage("Subject :"+aSubject+" property :"+sproperty+" object :"+sobject);
				//displayMessage("Not a leaf. ");
				results.close();
				qe.close();
				return false;
			}
		}
		results.close();
		qe.close();
		return true;
	}

	private boolean isResourceAsObjectShared(Model aModel,Resource aObject,String propertyURI){

				String queryString =
								"SELECT ?subject,?sproperty WHERE (?subject, <"+propertyURI+">, <"+aObject.getURI()+">)" +
								"USING vvasam FOR <" + mDocRDFNS + ">" ;
				//displayMessage("Query String :"+queryString);

				Query query = new Query(queryString);
				query.setSource(aModel);
				QueryExecution qe = new QueryEngine(query);
				QueryResults results = qe.exec();
				int count=0;
				while(results.hasNext()) {
					ResultBinding res = (ResultBinding)results.next() ;
					Value subject = res.getValue("subject");
					//displayMessage("subject :"+subject +" has :"+aObject+"  as a proprty :"+propertyURI);
					count++;
					if(count>1){
						results.close();
						qe.close();
						return true;
					}
				}

				results.close();
				qe.close();
				return false;

	}

	private boolean isResourceAsObjectShared(Model aModel,Resource aObject){

				String queryString =
								"SELECT ?subject,?sproperty WHERE (?subject, ?sproperty, <"+aObject.getURI()+">)" +
								"USING vvasam FOR <" + mDocRDFNS + ">" ;
				//displayMessage("Query String :"+queryString);

				Query query = new Query(queryString);
				query.setSource(aModel);
				QueryExecution qe = new QueryEngine(query);
				QueryResults results = qe.exec();
				int count=0;
				while(results.hasNext()) {
					ResultBinding res = (ResultBinding)results.next() ;
					Value subject = res.getValue("subject");
					Value prop = res.getValue("sproperty");
					displayMessage("subject :"+subject +" has :"+aObject+"  as a property :"+prop);
					count++;
					if(count>1){
						results.close();
						qe.close();
						return true;
					}
				}

				results.close();
				qe.close();
				return false;

	}


	private boolean isResourceAsObjectLinked(Model aModel,Resource aObject,String propertyURI){

					String queryString =
									"SELECT ?subject,?sproperty WHERE (?subject, <"+propertyURI+">, <"+aObject.getURI()+">)" +
									"USING vvasam FOR <" + mDocRDFNS + ">" ;
					//displayMessage("Query String :"+queryString);

					Query query = new Query(queryString);
					query.setSource(aModel);
					QueryExecution qe = new QueryEngine(query);
					QueryResults results = qe.exec();
					int count=0;
					if(results.hasNext()) {
						results.close();
						qe.close();
						return true;
					}else{
						results.close();
						qe.close();
						return false;
					}
	}


	/**
	 *This method will execute a query.
	 *@param inQueryString the query string
	 *@param inOutputBinVarName the bind variable name which will contain the output result.
	 *clause.
	 */
	public Resource[] getResource(String inQueryString, String inOutputBinVarName,Model aModel) throws Throwable {

		Resource retResources[] = null;
        Vector resources=new Vector();
		QueryResultsMem qryMem = executeQuery(inQueryString,aModel);
		while(qryMem.hasNext()) {
			ResultBinding resBinding = (ResultBinding)qryMem.next();
			Value value = resBinding.getValue(inOutputBinVarName);
			if (value==null){
				throw new RuntimeException("No resources are available for the given  bind variable " + inOutputBinVarName +
									" in the query " + inQueryString);
			}
			if( !value.isRDFResource() ) {
				throw new RuntimeException("The bind variable " + inOutputBinVarName +
					" is not of type 'Resource' in the query " + inQueryString);
			}

			resources.add(value.getRDFResource());
		}
		retResources=new Resource[resources.size()];
		retResources=(Resource[])resources.toArray(retResources);
		return retResources;
	}

	public QueryResultsMem executeQuery(String inQueryString,Model myModel) throws RDFNoDataFoundException {

		//long startTime = System.currentTimeMillis();

		Query query  = new Query(inQueryString);
		query.setSource(myModel);
		QueryExecution qe = new QueryEngine(query) ;
		QueryResults results = qe.exec() ;
		QueryResultsMem qrm = new QueryResultsMem(results);

		//displayMessage(">>>> Query Execution time in ms: " + (System.currentTimeMillis() - startTime));

		if ( 0 >= qrm.size() ) {
			throw new RDFNoDataFoundException("For query: \n" + inQueryString);
		}

		qe.close();

		return qrm;
	}

	private String readInput(String aDispalyMessage) throws Throwable{
		displayMessage(aDispalyMessage,false);
		String input=(mSysIn.readLine()).trim();
		return input;
	}

	private boolean getConfirmation(String aDispalyMessage) throws Throwable{
		String confirm=readInput(aDispalyMessage + " Do you want to continue(y/n)?");
		if(confirm!=null && confirm.equalsIgnoreCase("y")){
			return true;
		}else{
			return false;
		}

	}

	public  String getProperty(String aPropName) throws Throwable{
		return getProperty(aPropName, "");

	}

	public  String getProperty(String aPropName, int taskid) throws Throwable{
		return getProperty(aPropName, new Integer(taskid).toString());

	}

	public  String getProperty(String aPropName, String taskid) throws Throwable{

		System.out.println("                                     ");

		String modelName=mConfig.getProperty(aPropName);
		if(modelName==null || modelName.trim().equalsIgnoreCase("")){
			   System.out.println("    "+taskid+" --> The properties file has either no or empty "+ aPropName);
			   modelName=readInput("    "+taskid+" --> Enter value for "+aPropName+"  :");
		}
		if(modelName!=null && modelName.trim().equalsIgnoreCase("")){
		   System.out.println("    "+taskid+"----->Invalid "+ aPropName+"."+aPropName+" is blank.");
		   throw new RuntimeException("Invalid"+aPropName+".");
		}
		return modelName;
	 }

     private static void usage(){
        System.out.print("Usage:");
        System.out.println(" SemanticRDFUtils taskid ");
        System.out.println("   Where taskid  should be any one of the following:");
        System.out.println("    1 --> To create and initialize the Jena system tables with a system model name as JenaRDFSystem");
        System.out.println("    2 --> To create a database model");
        System.out.println("    3 --> To remove a database model.");
        System.out.println("    4 --> To list the contents of a given model.");
        System.out.println("    5 --> To import RDF/XML file to a database.");
        System.out.println("    6 --> To list existing database model names");
        System.out.println("    7 --> To export a database model to a RDF/XML file ");
		System.out.println("    8 --> To delete all the contents of a database model");
		System.out.println("    9 --> To create a union(RDF/XML file) of RDF/XML file models");
		System.out.println("    10 --> To create an intersection(RDF/XML file) of RDF/XML file models");
		System.out.println("    11 --> To create a difference(RDF/XML file) of RDF/XML file models");
		System.out.println("    12 --> To get the size of the given model");
		System.out.println("    13 --> Export the RDF query results as RDF/XML file.");
		System.out.println("    14 --> Delete the resource(s) from a model based on RDF query.");
        System.out.println(" ");

     }



     public static void main (String[] argv) throws Throwable
     {

        if(argv!=null && argv.length!=1){
             usage();
             System.exit(-1);
        }
        int taskid=0;
        String tid=argv[0];
        try{
           taskid=new Integer(tid).intValue();
           if(taskid<1 ||taskid>14 ){
               throw new Exception("Invalid taskid.");
           }
         }catch(Exception e){
            System.out.println("Invalid taskid .....");
            usage();
            System.exit(-1);
         }
        //SemanticRDFUtils rdbLoader=null;

		SemanticRDFUtils semRDFutils=null;
		PropertyFileReader config=null;
         try{
		    //rdbLoader=new SemanticRDFUtils();
		 	config= new PropertyFileReader("SemanticRDFUtils");
         	semRDFutils=new SemanticRDFUtils(config);
         }catch(Throwable te){
         	te.printStackTrace();
			System.exit(-1);
         }
         switch(taskid){
             case 1:{
                System.out.println(taskid+" --> Create and initialize the Jena system tables with a given system model name in the SemanticRDFUtils.properties");
				System.out.println("                                     ");
                 try{
					if(!semRDFutils.isRDFInDB()){
						throw new RuntimeException("This service is not for file RDF models.");
					}
					//String modelName=semRDFutils.getProperty(SemanticRDFUtils.RDF_SYSTEM_MODEL_NAME,taskid);
					//System.out.println("   ");
					semRDFutils.createAndInititalizeJenaTables();
                 }catch(Throwable e){
                     e.printStackTrace();
                 }
                 break;
             }
             case 2:{
                System.out.println(taskid+" --> Create a model with given name");
                try{
					if(!semRDFutils.isRDFInDB()){
						throw new RuntimeException("This service is not for file RDF models.");
					}
					String modelName=semRDFutils.getProperty(SemanticRDFUtils.MODELNAME,taskid);
					System.out.println("   ");
					semRDFutils.createRDBModel(modelName);
                }catch(Throwable e){
                     e.printStackTrace();
                }
                 break;
             }
             case 3:{
                System.out.println(taskid+" --> Remove a model with given name");
                try{
					if(!semRDFutils.isRDFInDB()){
						throw new RuntimeException("This service is not for file RDF models.");
					}
					String modelName=semRDFutils.getProperty(SemanticRDFUtils.MODELNAME,taskid);
					System.out.println("   ");
					semRDFutils.removeModel(modelName);
                 }catch(Throwable e){
                     e.printStackTrace();
                 }
                 break;
             }
             case 4:{
                System.out.println(taskid+"--> List contents of the given model");
				System.out.println("                                     ");
                 try{
					if(!semRDFutils.isRDFInDB()){
						semRDFutils.printRDBModel();
					}else{
						if(semRDFutils.isRDFDMModelInitialized()){
							semRDFutils.printRDBModel();
						}else{
						   String modelName=semRDFutils.getProperty(SemanticRDFUtils.MODELNAME,taskid);
						   semRDFutils.printRDBModel(modelName);
						}

					}

                 }catch(Throwable e){
                     e.printStackTrace();
                 }

                 break;
             }
             case 5:{
                 System.out.println(taskid+" --> To import RDF/XML file to a database.");
				 System.out.println("                                     ");
				 String rdfAbsFileName="";
                 try{
					if(!semRDFutils.isRDFInDB()){
						throw new RuntimeException("This service is not for file RDF models.");
					}
				   String modelName=semRDFutils.getProperty(SemanticRDFUtils.MODELNAME,taskid);
                   System.out.println(" ");
                   rdfAbsFileName=semRDFutils.getProperty(SemanticRDFUtils.IMPORT_RDFFILE_ABS_NAME,taskid);
				   semRDFutils.importRDFXMLFile(modelName,rdfAbsFileName);
                 }catch(Throwable e){
                     e.printStackTrace();
                 }
                 break;
             }
             case 6:{
                 System.out.println(taskid+"--> List existing model names");
				System.out.println("                                     ");
                 try{
					if(!semRDFutils.isRDFInDB()){
						throw new RuntimeException("This service is not for file RDF models.");
					}
					semRDFutils.listExistingModels();
                 }catch(Throwable e){
                     e.printStackTrace();
                 }
                 break;
             }
             case 7:{
                 System.out.println(taskid+" -->To export a database model to a RDF/XML file");
				 System.out.println("                                     ");
                 String rdfFileName="";
                 try{
					if(!semRDFutils.isRDFInDB()){
						throw new RuntimeException("This service is not for file RDF models.");
					}else{
						if(semRDFutils.isRDFDMModelInitialized()){
							rdfFileName=semRDFutils.getProperty(SemanticRDFUtils.EXPORT_RDFFILE_ABS_NAME,taskid);
							semRDFutils.exportRDFDBModelToRDFXMLFile(rdfFileName);
						}else{
						   String modelName=semRDFutils.getProperty(SemanticRDFUtils.MODELNAME,taskid);
						   rdfFileName=semRDFutils.getProperty(SemanticRDFUtils.EXPORT_RDFFILE_ABS_NAME,taskid);
						   semRDFutils.exportRDFDBModelToRDFXMLFile(modelName,rdfFileName);
						}
					}

                 }catch(Throwable e){
                     e.printStackTrace();
                 }

                 break;
             }

			case 8:{
				System.out.println(taskid+"--> Delete all the contents of a given model");
				System.out.println("                                     ");
				try{
					if(!semRDFutils.isRDFInDB()){
						throw new RuntimeException("This service is not for file RDF models.");
					}else{
						if(semRDFutils.isRDFDMModelInitialized()){
							semRDFutils.deleteModelContents();
						}else{
						   String modelName=semRDFutils.getProperty(SemanticRDFUtils.MODELNAME,taskid);
						   semRDFutils.deleteModelContents(modelName);
						}

					}
				}catch(Throwable e){
				   e.printStackTrace();
				}
				break;
			}
			case 9:{
				System.out.println(taskid+" --> To create a union(RDF/XML file) of RDF/XML file models");
				System.out.println("                                     ");
				try{
					//semRDFutils.saveUnionModelToFile();
					semRDFutils.saveGraphOperationResultModelToFile(SemanticRDFUtils.MODEL_UNION);
				}catch(Throwable e){
  					e.printStackTrace();
				}
				break;
			}
			case 10:{
				System.out.println(taskid+" --> To create an intersection(RDF/XML file) of RDF/XML file models");
				System.out.println("                                     ");
				try{
					//semRDFutils.saveIntersectionModelToFile();
					semRDFutils.saveGraphOperationResultModelToFile(SemanticRDFUtils.MODEL_INTERSECTION);
				}catch(Throwable e){
				  e.printStackTrace();
				}
				break;
			}
			case 11:{
				System.out.println(taskid+" --> To create a difference(RDF/XML file) of RDF/XML file models");
				System.out.println("                                     ");
				try{
					//semRDFutils.saveDifferenceModelToFile();
					semRDFutils.saveGraphOperationResultModelToFile(SemanticRDFUtils.MODEL_DIFFERENCE);
				}catch(Throwable e){
					e.printStackTrace();
				}
				break;
		   }

		   case 12:{
			 System.out.println(taskid+" --> Get the size of the given model");
			 System.out.println("                                     ");
			 long size=0;
			 String modelName="";
			 try{
			   if(!semRDFutils.isRDFInDB()){
				size=semRDFutils.getStmtCount();
			   }else{
				   if(semRDFutils.isRDFDMModelInitialized()){
					size=semRDFutils.getStmtCount();
				   }else{
					  modelName=semRDFutils.getProperty(SemanticRDFUtils.MODELNAME,taskid);
					  size=semRDFutils.getStmtCount(modelName);
				   }

			   }
			  System.out.println("			----->"+modelName+" size is :"+size+" rdf statements");
			}catch(Throwable e){
				e.printStackTrace();
			}
			break;
		}

		case 13:{
		  System.out.println(taskid+" --> Export the RDF query results as RDF/XML file.");
		  System.out.println("                                     ");
		  String modelName="";
		  String rdfFileName="";
		  String rdf_query="";
		  String bind_var_name="";
		  try{
		   System.out.println(taskid+"-->Querying and creating a RDF/XML file ....");
			  if(!semRDFutils.isRDFInDB()){
				rdfFileName=semRDFutils.getProperty(SemanticRDFUtils.EXPORT_RDFFILE_ABS_NAME,taskid);
				rdf_query= semRDFutils.getProperty(SemanticRDFUtils.RDF_QUERY,taskid);
				bind_var_name=semRDFutils.getProperty(SemanticRDFUtils.BIND_VAR_NAME,taskid);
				semRDFutils.exportRDFQueryResultsAsRDFXMLFile(rdfFileName,rdf_query,bind_var_name);
			  }else{
				  if(semRDFutils.isRDFDMModelInitialized()){
					rdfFileName=semRDFutils.getProperty(SemanticRDFUtils.EXPORT_RDFFILE_ABS_NAME,taskid);
					rdf_query= semRDFutils.getProperty(SemanticRDFUtils.RDF_QUERY,taskid);
					bind_var_name=semRDFutils.getProperty(SemanticRDFUtils.BIND_VAR_NAME,taskid);
					semRDFutils.exportRDFQueryResultsAsRDFXMLFile(rdfFileName,rdf_query,bind_var_name);

				  }else{
					modelName=semRDFutils.getProperty(SemanticRDFUtils.MODELNAME,taskid);
					rdfFileName=semRDFutils.getProperty(SemanticRDFUtils.EXPORT_RDFFILE_ABS_NAME,taskid);
					rdf_query= semRDFutils.getProperty(SemanticRDFUtils.RDF_QUERY,taskid);
					bind_var_name=semRDFutils.getProperty(SemanticRDFUtils.BIND_VAR_NAME,taskid);
					semRDFutils.exportRDFQueryResultsAsRDFXMLFile(modelName,rdfFileName,rdf_query,bind_var_name);
				  }

			  }
		  }catch(Throwable e){
			  e.printStackTrace();
		  }
		  break;

	 	}

	 	case 14:{
	   		System.out.println(taskid+" --> Delete the resource(s) from a model based on RDF query.");
	   		System.out.println("                                     ");
			String modelName="";
			String rdf_query="";
			String bind_var_name="";
			try{
			 System.out.println(taskid+"-->Querying and creating a RDF/XML file ....");
				if(!semRDFutils.isRDFInDB()){
					throw new RuntimeException("This service is not for file RDF models.");
				}else{
					if(semRDFutils.isRDFDMModelInitialized()){
					  rdf_query= semRDFutils.getProperty(SemanticRDFUtils.RDF_QUERY,taskid);
					  bind_var_name=semRDFutils.getProperty(SemanticRDFUtils.BIND_VAR_NAME,taskid);
					  semRDFutils.deleteRDFQueryResultsFromModel(rdf_query,bind_var_name);

					}else{
					  modelName=semRDFutils.getProperty(SemanticRDFUtils.MODELNAME,taskid);
					  rdf_query= semRDFutils.getProperty(SemanticRDFUtils.RDF_QUERY,taskid);
					  bind_var_name=semRDFutils.getProperty(SemanticRDFUtils.BIND_VAR_NAME,taskid);
					  semRDFutils.deleteRDFQueryResultsFromModel(modelName,rdf_query,bind_var_name);
					}

				}
			}catch(Throwable e){
				e.printStackTrace();
			}
			break;

	  	}

        default:{
                System.out.println("Invalid taskid .....");
                usage();
                System.exit(-1);
                break;
             }
         }
		System.out.println("                                     ");
         try{
			semRDFutils.close();
         }catch(Exception ex){
             ex.printStackTrace();
             System.exit(-1);
         }
         System.exit(0);

    }


}

